% Generating the Polar Fractal
% B.C.Geiger, LNT/TUM
% (c) 2015

clear all; close all; clc;

g(1,:)=[-1 2 0];    % g_0(x)=2x-x^2=1-(1-x)^2
g(2,:)=[1 0 0];     % g_1(x)=x^2


basis=[0 0 1 1];    % set the recurring pattern for 1/5


r=[1 0];            % initial polynomial is x
for ind=1:length(basis)
    r=polyComp(r,g(basis(ind)+1,:));    % polynomial composition according to recurring pattern
end
r(end-1)=-1;        % subtract x to set up equation p_n(x)-x
r_roots=roots(r);   % determine roots
r_roots=r_roots(imag(roots(r))==0 & roots(r)<1-1e-6 & roots(r)>0);  % determine real root(s) inside the unit interval

% The real root of inside the unit interval determines the threshold t: if
% Z(W)<t, then the channel gets perfect.

BaseMat(1,:)=[0,1];     % dyadic rational

BaseMat(2,1)=1/5;       % t for 1/5
BaseMat(2,2)=r_roots;

BaseMat(3,1)=2/5;       % t for 2/5 = 1- t for 3/5
BaseMat(3,2)=1-sqrt(r_roots);

BaseMat(4,1)=3/5;       % 3/5 = 1/2+1/5/2, Z^2(W)<t => Z(W)<sqrt(t)
BaseMat(4,2)=sqrt(r_roots);

BaseMat(5,1)=4/5;       % t for 4/5 = 1- t for 1/5
BaseMat(5,2)=1-r_roots;

BaseMat(6,:)=[1,1];     % dyadic rational

for ind=1:7
    laenge=size(BaseMat,1);
    % Dividing by two means shift of the pattern to the right, prepending a
    % 0. 1-(1-Z(W))^2<t => Z(W) < 1-sqrt(1-t)
    BaseMat(laenge+1:2*laenge,1)=BaseMat(1:laenge,1)/2;
    BaseMat(laenge+1:2*laenge,2)=1-sqrt(1-BaseMat(1:laenge,2));
    
    % Dividing by two and adding 0.5 means prepending a 1. Z(W)^2<t =>
    % Z(W)<sqrt(t)
    BaseMat(2*laenge+1:3*laenge,1)=BaseMat(1:laenge,1)/2+0.5;
    BaseMat(2*laenge+1:3*laenge,2)=sqrt(BaseMat(1:laenge,2));
    
    % remove duplicates
    BaseMat=sortrows(unique(BaseMat,'rows'));
    laenge=size(BaseMat,1);
    difference=diff(BaseMat(:,1));
    BaseMat(abs(difference)<1e-6,:)=[];
end

figure
subplot(3,1,1)
stem(2*BaseMat(1:ceil(end/2),1),BaseMat(1:ceil(end/2),2))
subplot(3,1,2)
stem(BaseMat(1:2:end,1),BaseMat(1:2:end,2))
subplot(3,1,3)
stem(2*BaseMat(floor(end/2)+1:end,1)-1,BaseMat(floor(end/2)+1:end,2))


fid=fopen('Base.dat', 'w');

for ind=1:size(BaseMat,1)
    fprintf(fid, '%f, %f\n', BaseMat(ind,1),BaseMat(ind,2));
end

fclose(fid);